/*
 *  $Id: ElementMapRenderer.java,v 1.5 2001/11/28 20:36:20 fekete Exp $
 *  Copyright (C) 1998 Jean-Daniel Fekete and Ecole des Mines de Nantes
 *
 *  This file is part of Compus.
 *  Compus is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Library General Public License as published
 *  by the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import javax.swing.*;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;


public class ElementMapRenderer extends JLabel implements ListCellRenderer {
  //    static Font font = new Font("Serif", Font.PLAIN, 20);
  static class ColorIcon implements Icon {
    int size;
    Color color;
    char code;
    ColorIcon(int size, Color color, char code) {
      this.size = size;
      this.color = color;
      this.code = code;
    }
    public void paintIcon(Component c, Graphics g, int x, int y) {
      g.setColor(Color.black);
      g.drawRect(x, y, size, size);
      FontMetrics fm = g.getFontMetrics();
      g.drawString(code+"",
		   x+size+(size - fm.charWidth(code))/2,
		   y + fm.getAscent());
      g.setColor(color);
      g.fillRect(x+1, y+1, size-1, size-1);
    }
    public int getIconWidth() { return 2*size; }
    public int getIconHeight() { return size; }
  }
  ElementMap map;
  public ElementMapRenderer(ElementMap map) {
    this.map = map;
    setOpaque(true);
  }
  public Component getListCellRendererComponent(
						JList list,
						Object value,
						int index,
						boolean isSelected,
						boolean cellHasFocus)
    {
      String name = value.toString();
      ElementType type = map.getType(name);
      Font font = list.getFont();

      setFont(font);
      setText(name);
      char code;
      if (type != null) {
	code = type.getTypeChar();
      } else {
	code = ' ';
      }
      Color c = map.getColor(name);
      if (c.getAlpha() != 255) {
	c = new Color(c.getRed(), c.getGreen(), c.getBlue());	// get rid of alpha
      }
      setIcon(new ColorIcon(font.getSize(), c, code));
      if (isSelected) {
	setBackground(list.getSelectionBackground());
	setForeground(list.getSelectionForeground());
	//setBackground(Color.red);
	//setForeground(Color.white);
      } else {
	setBackground(list.getBackground());
	setForeground(list.getForeground());
	//setBackground(Color.white);
//  	}
//  	setForeground(c);
      }
      setEnabled(list.isEnabled());
      return this;
    }
}
