/*
 *  $Id: RulerBorder.java,v 1.2 2001/11/28 20:36:20 fekete Exp $
 *  Copyright (C) 1998 Jean-Daniel Fekete and Ecole des Mines de Nantes
 *
 *  This file is part of Compus.
 *  Compus is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Library General Public License as published
 *  by the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import javax.swing.border.*;

import java.awt.Container;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.Component;
import java.awt.Dimension;

import javax.swing.UIManager;

public class RulerBorder extends AbstractBorder
{
  protected Color  titleColor;

  // Space between the border and the component's edge
  static public final int EDGE_HEIGHT = 7;

  public RulerBorder() {
    this(null);
  }
  public RulerBorder(Color titleColor) {
    this.titleColor = titleColor;
  }

  public void paintBorder(Component c, Graphics g, int x, int y, int width, int height) {
    Container cont = (Container)c;
    int size = cont.getComponentCount();
    if (size == 0) {
      return;
    }

    Color color = g.getColor();

    Insets insets = new Insets(EDGE_HEIGHT, 0, 0, 0);

    g.setColor(getTitleColor());

    int i;
    int widths[] = new int[size];
    for (i = 0; i < size; i++) {
      widths[i] = cont.getComponent(i).getSize().width;
    }

    for (i = 0; i < widths.length; i++) {
      int cx = x;
      if ((i % 10) == 0) {
	g.drawLine(cx, y, cx, y+7);
      }
      else if ((i % 5) == 0) {
	g.drawLine(cx, y+2, cx, y+7);
      }
      else {
	g.drawLine(cx, y+4, cx, y+7);
      }
      x += widths[i];
    }

    g.setColor(color);
  }

  public boolean isBorderOpaque() { return true; }

  /**
   * Returns the title-color of the titled border.
   */
  public Color getTitleColor()    {       
    Color c = titleColor;
    if (c == null)
      c = UIManager.getColor("TitledBorder.titleColor");
    return c;  
  }

  /**
   * Sets the title-color of the titled border.
   * @param titleColor the color for the border title
   */
  public void setTitleColor(Color titleColor) {       
    this.titleColor = titleColor;   
  }

  public Dimension getMinimumSize(Component c) {
    Insets insets = getBorderInsets(c);
    return new Dimension(insets.right+insets.left, 
			 insets.top+insets.bottom+
			 EDGE_HEIGHT);
  }

  public Insets getBorderInsets(Component c) {
    return new Insets(EDGE_HEIGHT, 0, 0, 0);
  }
    public String getToolTipText(Component c, int x, int y) {
	if (y > EDGE_HEIGHT)
	    return null;
	Container cont = (Container)c;
	int size = cont.getComponentCount();
	if (size == 0) {
	    return null;
	}

	int i;
	
	for (i = 0; i < size && x >= 0; i++) {
	    x -= cont.getComponent(i).getSize().width;
	}
	//System.out.println("mouse pointing col "+i);
	return "Column "+i;
    }
}

