/*
 *  $Id: SNSResult.java,v 1.4 2001/11/28 20:36:20 fekete Exp $
 *  Copyright (C) 1998 Jean-Daniel Fekete and Ecole des Mines de Nantes
 *
 *  This file is part of Compus.
 *  Compus is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Library General Public License as published
 *  by the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import com.jclark.xsl.sax.*;
import com.jclark.xsl.om.*;
import com.jclark.xsl.tr.*;
import org.xml.sax.*;
import java.io.IOException;

class SNSResult implements Result, AttributeList {
  private DocumentHandler documentHandler;
  SNSResult(DocumentHandler handler) {
    documentHandler = handler;
  }

  public void rawCharacters(String str) throws XSLException {
    characters(str);
  }

  public void characters(String str) throws XSLException {
    char buf[] = str.toCharArray();
    try {
      getDocumentHandler().characters(buf, 0, buf.length);
    } 
    catch (SAXException e) {
      throwXSLException(e);
    }
  }
  // The nsMap must declare the prefix on elementType correctly.
  public void comment(String str) throws XSLException { }
  public void processingInstruction(String target, String data) throws XSLException { }
  public void attribute(Name name, String value) throws XSLException { }
  public void start(OutputMethod outputMethod) throws XSLException {
    try {
      getDocumentHandler().startDocument();
    }
    catch (SAXException e) {
      throwXSLException(e);
    }
}
  public void end() throws XSLException {
    try {
      getDocumentHandler().endDocument();
    }
    catch (SAXException e) {
      throwXSLException(e);
    }
  }
  public void message(Node node, String str) throws XSLException { }
  public Result createResult(String uri) throws XSLException {
    return null;
  }

  public DocumentHandler getDocumentHandler() { return documentHandler; }

  public void startElement(Name elementType,
			  NamespacePrefixMap nsMap)
    throws XSLException {
    try {
      getDocumentHandler().startElement(elementType.getLocalPart(),
					this);
    }
    catch (SAXException e) {
      throwXSLException(e);
    }
  }

  public void endElement(Name elementType) throws XSLException {
    try {
      getDocumentHandler().endElement(elementType.getLocalPart());
    }
    catch (SAXException e) {
      throwXSLException(e);
    }
  }

  public String getName(int i) {
    return "";
  }

  public void resultTreeFragment(ResultTreeFragment frag) throws XSLException {
    try {
      frag.emit(getDocumentHandler());
    }
    catch (SAXException e) {
      throwXSLException(e);
    }
  }
  public int getLength() {
    return 0;
  }

  protected final Name getAttributeName(int i) {
    return null;
  }

  public String getValue(int i) {
    return null;
  }

  public String getType(int i) {
    return "CDATA";
  }

  public String getType(String name) {
    return "CDATA";
  }

  public String getValue(String name) {
    return null;
  }
  protected void throwXSLException(SAXException e) throws XSLException {
    Exception wrapped = e.getException();
    if (wrapped != null)
      throw new XSLException(wrapped);
    else
      throw new XSLException(e.getMessage());
  }
}
