/*
 *  $Id: XSLDocument.java,v 1.4 2001/11/28 20:36:20 fekete Exp $
 *  Copyright (C) 1998 Jean-Daniel Fekete and Ecole des Mines de Nantes
 *
 *  This file is part of Compus.
 *  Compus is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Library General Public License as published
 *  by the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import java.util.Vector;
import com.jclark.xsl.sax.*;
import org.xml.sax.*;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.File;
import java.io.FileDescriptor;
import java.net.URL;

public class XSLDocument extends XmlDocument {
  String sheetName;

  public XSLDocument(String name, String sheetName) {
    super();
    systemId = name;
    this.sheetName = sheetName;
    try {
      load();
    } catch(Exception e) {
      System.out.println(e);
    }
  }
  public XSLDocument(String name, Parser parser, InputSource source, String sheetName) {
    super();
    systemId = name;
    this.sheetName = sheetName;
    try {
      load(parser, source);
    } catch(Exception e) { 
      System.out.println(e);
    }
  }

  public XSLDocument(String name, XSLProcessorImpl xsl, InputSource inputSource) {
    super();
    systemId = name;
    this.sheetName = sheetName;
    try {
      load(xsl, inputSource);
    } catch(Exception e) { 
      System.out.println(e);
    }
  }

  public synchronized void load(Parser sourceParser, InputSource inputSource) 
    throws Exception {
    load(sourceParser, inputSource, createParser(), fileInputSource(sheetName));
  }

  public synchronized void load(Parser sourceParser, InputSource inputSource,
				Parser sheetParser, InputSource sheetSource) 
    throws Exception {
    loaded = false;
    opened = new Vector();
    elements = new Vector();
    events = new Vector();

    //System.out.println("Ready to load "+systemId+" with sylesheet "+sheetName);
    XSLProcessorImpl xsl = new XSLProcessorImpl();
    xsl.setParser(sourceParser, sheetParser);

    xsl.loadStylesheet(sheetSource);
    xsl.setDocumentHandler(this);
    xsl.parse(inputSource);
    loaded = true;
    opened = null;
    elements.trimToSize();
    events.trimToSize();
  }

  public synchronized void load(XSLProcessorImpl xsl, InputSource inputSource)
    throws Exception {
    loaded = false;
    opened = new Vector();
    elements = new Vector();
    events = new Vector();

    //System.out.println("Ready to load "+systemId+" with an XSL Processor");
    xsl.setDocumentHandler(this);
    xsl.parse(inputSource);
    loaded = true;
    opened = null;
    elements.trimToSize();
    events.trimToSize();
  }

  

}
