/*
 *  $Id: XSLPanel.java,v 1.4 2001/11/28 20:36:20 fekete Exp $
 *  Copyright (C) 1998 Jean-Daniel Fekete and Ecole des Mines de Nantes
 *
 *  This file is part of Compus.
 *  Compus is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Library General Public License as published
 *  by the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

public class XSLPanel extends Notepad {
  JComboBox combo;
  XmlCompare comp;
  Hashtable namedsheets;
  String current;

  public XSLPanel(XmlCompare compare) {
    comp = compare;
    namedsheets = new Hashtable();
    selectStylesheet("Stylesheet1");
  }

  protected Component createStatusbar() {
    Box b = Box.createHorizontalBox();
    combo = new JComboBox();
    combo.setEditable(true);
    combo.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
	if (e.getStateChange() == ItemEvent.SELECTED) {
	  String name = (String)e.getItem();
	  addStylesheet(name);
	  selectStylesheet(name);
	}
      }
    });
    JButton apply = new JButton("Apply");
    apply.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	String text =
	  //"<xsl:stylesheet xmlns:xsl='http://www.w3.org/XSL/Transform'>\n"+
	  getEditor().getText() +
	  "";
	  //"\n</xsl:stylesheet>\n";
	
	String name = getName();
	//System.out.println("Name = "+name);
	if (name != null && ! current.equals(name)) {
	  addStylesheet(name);
	  selectStylesheet(name);
	  getEditor().setText(text);
	}
	  
	comp.updateNewViews((String)combo.getSelectedItem(), text);
      }
    });
    b.add(combo);
    b.add(apply);
    b.add(super.createStatusbar());
    return b;
  }
  protected JFileChooser createFileChooser() {
    if (fileChooser == null) {
      super.createFileChooser();
      ExampleFileFilter filter =
	new ExampleFileFilter(new String[]{"xsl"}, "XSL Files");
      fileChooser.addChoosableFileFilter(filter);
    }
    return fileChooser;
  }

  public DefaultComboBoxModel getComboBoxModel() {
    return (DefaultComboBoxModel)combo.getModel();
  }
  public void selectStylesheet(String name) {
    if (current != null) {
      namedsheets.put(current, getEditor().getDocument());
    }
    current = name;
    //System.out.println("Selecting "+name);
    getComboBoxModel().setSelectedItem(name);
    getEditor().setDocument((PlainDocument)namedsheets.get(name));
  }

  public void addStylesheet(String name) {
    if (namedsheets.containsKey(name))
      return;
    DefaultComboBoxModel cbm = getComboBoxModel();
    cbm.addElement(name);
    namedsheets.put(name, new PlainDocument());
  }

}
