/*
 *  $Id: XmlCompare.java,v 1.12 2001/11/28 20:36:20 fekete Exp $
 *  Copyright (C) 1998 Jean-Daniel Fekete and Ecole des Mines de Nantes
 *
 *  This file is part of Compus.
 *  Compus is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Library General Public License as published
 *  by the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import org.xml.sax.*;
import java.util.*;
import java.io.*;
import com.jclark.xsl.sax.*;
import com.jclark.xsl.tr.*;
import com.jclark.xsl.om.*;
import java.util.Locale;
import java.io.IOException;
import java.net.URL;

public class XmlCompare extends JFrame implements ParameterSet
  //  implements ListSelectionListener
{
  Node nodes[];
  int maxSize = 0;
  ElementMap elementMap = null;
  ElementMap selementMap = null;
  JTextArea text;
  // String stylesheet = "diplo-xt.xsl";
  JTabbedPane views /* , texts */;
  JProgressBar pbar;
  XMLProcessorImpl xml;
  XSLProcessorImpl xsl;
  Engine engine;
  int order[];
  private Hashtable params = new Hashtable();
  int display_type = XmlViewBase.DISPLAY_DOC_ORDER;
  //JDialog dialog;
  JTextField textDialog;

  XmlCompare(String args[]) throws Exception {
    super("Compus");
    RepaintManager repaintManager = 
      RepaintManager.currentManager(this);
    if(repaintManager.isDoubleBufferingEnabled()) {
      repaintManager.setDoubleBufferingEnabled(false);
    }
    addWindowListener( new WindowAdapter() {
      public void windowClosing(WindowEvent e) {System.exit(0);}
    });

    //getContentPane().setLayout(new FlowLayout(FlowLayout.LEFT, 1, 1));
    getContentPane().setLayout(new BorderLayout());

    getContentPane().add(createMenuBar(), BorderLayout.NORTH);
    views = new JTabbedPane();
    getContentPane().add(views, BorderLayout.CENTER);
    //texts = new JTabbedPane();
    //getContentPane().add(texts, BorderLayout.EAST);
    pbar = new JProgressBar(JProgressBar.VERTICAL);
    pbar.setMinimum(0);
    getContentPane().add(pbar, BorderLayout.WEST);

    int i;
    Parser parser = XmlXtView.createParser();
    xml = new XMLProcessorImpl(parser);
    engine = new EngineImpl(xml, null);

    nodes = new Node[args.length];
    for (i = 0; i < args.length; i++) {
      System.out.print("Loading "+args[i]+"...");
      nodes[i] = xml.load(XmlXtView.fileInputSource(args[i]),
			  i,
			  engine.getSheetLoadContext(),
			  engine.getNameTable());
      System.out.println("done.");
    }
    
    text = new JTextArea();
    //text.setPreferredSize(new Dimension(100, 800));
    text.setLineWrap(true);
    text.setWrapStyleWord(true);
    text.setEditable(false);
    XmlXtViews v = new XmlXtViews("Original", nodes, text);
    order = v.getSortOrder("Original");
    v.setMaxSize(v.getMaxSize());
    views.addTab("Original", null, v);

    /*
    texts.addTab("Work", null,
		 new JScrollPane(text,
				 JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				 JScrollPane.HORIZONTAL_SCROLLBAR_NEVER));
    texts.addTab("Stylesheet", null, new XSLPanel(this));
    texts.setPreferredSize(new Dimension(300, 700));
    */
    views.addTab("Work", null,
		 new JScrollPane(text,
				 JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				 JScrollPane.HORIZONTAL_SCROLLBAR_NEVER));
    views.addTab("Stylesheet", null, new XSLPanel(this));
    pack();
    setVisible(true);
    //dialog = new JDialog(XmlCompare.this, "Find...");
    //dialog.setLayout(new BorderLayout());
    textDialog = new JTextField(50);
    //dialog.getContentPane().add(text, BorderLayout.CENTER);
    textDialog.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	getCurrentViews().match(e.getActionCommand());
      }
    });
    /*
    JButton but = new JButton("Erase");
    dialog.getContentPane().add(but, BorderLayout.SOUTH);
    but.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	textDialog.setText("");
	getCurrentViews().match(null);
      }
    });
    dialog.addWindowListener( new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
	dialog.setVisible(false);
      }
    });
    dialog.pack();
    */
  }

  JMenuBar createMenuBar() {
    JMenuBar menuBar = new JMenuBar();
    JMenuItem mi;

    // File Menu
    JMenu file = (JMenu) menuBar.add(new JMenu("File"));
    file.setMnemonic('F');
    mi = (JMenuItem) file.add(new JMenuItem("Open Stylesheet..."));
    mi.setMnemonic('O');
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	JFileChooser chooser = new JFileChooser(".");
	ExampleFileFilter filter = new ExampleFileFilter(new String[]{"xsl"}, "XSL Files");
	chooser.addChoosableFileFilter(filter);
	int retval = chooser.showOpenDialog(XmlCompare.this);
	if (retval == 0) {
	  File f = chooser.getSelectedFile();
	  if (f != null) {
	    String name;
	    if (f.isAbsolute())
	      name = f.getAbsolutePath();
	    else
	      name = f.getName();
	    updateNewViews(name);
	  }
	}
      }
    });
//      mi = (JMenuItem) file.add(new JMenuItem("Find..."));
//      mi.setMnemonic('f');
//      mi.addActionListener(new ActionListener() {
//        public void actionPerformed(ActionEvent e) {
//  	dialog.setVisible(true);
//        }
//      });
    mi = (JMenuItem) file.add(new JMenuItem("Export..."));
    mi.setMnemonic('e');
    mi.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  XmlViewsBase base = getCurrentViews();
	  if (base == null)
	    return;
	  JFileChooser chooser = new JFileChooser(".");
	  int retval = chooser.showOpenDialog(XmlCompare.this);
	  if (retval != 0) {
	    return;
	  }
	  File f = chooser.getSelectedFile();
	  if (f == null) {
	    return;
	  }
	  String name;
	  if (f.isAbsolute())
	    name = f.getAbsolutePath();
	  else
	    name = f.getName();

	  base.saveViews(name);
        }
      });
    
    file.addSeparator();
    mi = (JMenuItem) file.add(new JMenuItem("Exit"));
    mi.setMnemonic('x');
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	System.exit(0);
      }
    });

    JMenu view = (JMenu)menuBar.add(new JMenu("View"));
    view.setMnemonic('V');
    JRadioButtonMenuItem rmi;
    ButtonGroup group = new ButtonGroup();
    rmi = (JRadioButtonMenuItem)
      view.add(new JRadioButtonMenuItem("Original Order"));
    rmi.setMnemonic('o');
    rmi.setSelected(true);
    rmi.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
	if (e.getStateChange() == ItemEvent.SELECTED) {
	  XmlViewsBase v = getCurrentViews();
	  if (v != null) {
	    order = v.getSortOrder("Original");
	    sortViews();
	  }
	}
      }
    });
    group.add(rmi);
    String sorters[] = XmlViewsBase.getViewCompNames();
    for (int i = 0; i < sorters.length; i++) {
      rmi = (JRadioButtonMenuItem)
	view.add(new JRadioButtonMenuItem("Sort by "+sorters[i]));
      rmi.addItemListener(new ViewsSorter(sorters[i]));
      group.add(rmi);
    }

    view.addSeparator();

    rmi = (JRadioButtonMenuItem)
      view.add(new JRadioButtonMenuItem("Sort Elements"));
    rmi.addItemListener(new ItemListener() {
	public void itemStateChanged(ItemEvent e) {
	  display_type = (e.getStateChange() == ItemEvent.SELECTED) ?
	    XmlViewBase.DISPLAY_SORTED_ORDER :
	    XmlViewBase.DISPLAY_DOC_ORDER;
	  setDisplayType();
	}
      });

    JMenu colors = (JMenu)menuBar.add(new JMenu("Colors"));
    view.setMnemonic('C');
    mi = (JMenuItem) colors.add(new JMenuItem("Optimize"));
    mi.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  XmlViewsBase v = getCurrentViews();
	  if (v != null)
	    v.optimizeColors();
	}
      });

    final JCheckBoxMenuItem check = new JCheckBoxMenuItem("Invert Selection",
							  false);
    mi = (JMenuItem) colors.add(check);
    mi.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  XmlViewsBase v = getCurrentViews();
	  if (v != null)
	    v.setSelectionInverted(check.isSelected());
	}
      });
			 
    textDialog = new JTextField(50);
    menuBar.add(textDialog);
    textDialog.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	getCurrentViews().match(e.getActionCommand());
      }
    });

    return menuBar;
  }

  class ViewsSorter implements ItemListener {
    String sorter;

    ViewsSorter(String s) { sorter = s; }
    public void itemStateChanged(ItemEvent e) {
      if (e.getStateChange() == ItemEvent.SELECTED) {
	XmlViewsBase v = getCurrentViews();
	if (v != null) {
	  order = v.getSortOrder(sorter);
	  sortViews();
	}
      }
    }
  }

  public void updateNewViews(String sheetname) {
    Thread updater =
      new NewViewsUpdater(sheetname, XmlDocument.fileInputSource(sheetname));
    updater.start();
    //updateNewViews(sheetname, XmlDocument.fileInputSource(sheetname));
  }
  public void updateNewViews(String sheetname, String xsl) {
    Thread updater = 
      new NewViewsUpdater(sheetname, new InputSource(new StringReader(xsl)));
    updater.start();
    //updateNewViews(sheetname, new InputSource(new StringReader(xsl)));
  }
  public void updateNewViews(String sheetname, InputSource source) {
    File sfile = new File(sheetname);
    String sname = sfile.getName();
    Node node;
    Sheet sheet;
    try {
      node = xml.load(source,
		      0,
		      engine.getSheetLoadContext(),
		      engine.getNameTable());
      sheet = engine.createSheet(node);

      XmlDocument new_docs[] = new XmlDocument[nodes.length];
      pbar.setMaximum(nodes.length);
      for (int i = 0; i < nodes.length; i++) {
	File file = new File(nodes[i].getURL().toString());
	String name = sname+"_"+file.getName();
	System.out.println("Processing "+name);
	new_docs[i] = new XmlDocument();
	new_docs[i].setName(name);
	sheet.process(nodes[i], xml, this, new SNSResult(new_docs[i]));
	pbar.setValue(i);
      }

      int index = views.indexOfTab(sname);

      XmlDocumentViews v;
      if (index == -1) {
	v = new XmlDocumentViews(sname, new_docs, text);
	v.setMaxSize(v.getMaxSize());
	views.addTab(sname, null, v);
	index = views.indexOfTab(sname);
      } else {
	v = (XmlDocumentViews)views.getComponentAt(index);
	v.setDocuments(new_docs);
	v.setMaxSize(v.getMaxSize());
      }
      views.setSelectedIndex(index);
      pbar.setValue(0);
    } catch (XSLException e) {
      String systemId = null;
      int lineNumber = -1;
      Node n = e.getNode();
      if (n != null) {
	URL url = n.getURL();
	if (url != null)
	  systemId = url.toString();
	lineNumber = n.getLineNumber();
      }
      Exception wrapped = e.getException();
      String message = e.getMessage();
      if (systemId != null || lineNumber != -1)
	System.out.println(systemId + lineNumber + message );
      return;
    }
    catch(IOException e) {
      System.out.println("IO Exception"+e);
      return;
    }
  }
  class NewViewsUpdater extends Thread {
    String sheetname;
    InputSource source;
    NewViewsUpdater(String sheetname, InputSource source) {
      this.sheetname = sheetname;
      this.source = source;
    }
    public void run() {
      File sfile = new File(sheetname);
      String sname = sfile.getName();
      Node node;
      Sheet sheet;
      try {
	node = xml.load(source,
			0,
			engine.getSheetLoadContext(),
			engine.getNameTable());
	sheet = engine.createSheet(node);

	XmlDocument new_docs[] = new XmlDocument[nodes.length];
	pbar.setMaximum(nodes.length);
	for (int i = 0; i < nodes.length; i++) {
	  File file = new File(nodes[i].getURL().toString());
	  String name = sname+"_"+file.getName();
	  System.out.println("Processing "+name);
	  new_docs[i] = new XmlDocument();
	  new_docs[i].setName(name);
	  sheet.process(nodes[i], xml, XmlCompare.this,
			new SNSResult(new_docs[i]));
	  pbar.setValue(i);
	}

	SwingUtilities.invokeLater(new NewViewsRunnable(sname, new_docs));
	pbar.setValue(0);
      } catch (XSLException e) {
	String systemId = null;
	int lineNumber = -1;
	Node n = e.getNode();
	if (n != null) {
	  URL url = n.getURL();
	  if (url != null)
	    systemId = url.toString();
	  lineNumber = n.getLineNumber();
	}
	Exception wrapped = e.getException();
	String message = e.getMessage();
	if (systemId != null || lineNumber != -1)
	  System.out.println(systemId + lineNumber + message );
	return;
      }
      catch(IOException e) {
	System.out.println("IO Exception"+e);
	return;
      }
    }
  }
  class NewViewsRunnable implements Runnable {
    String sname;
    XmlDocument new_docs[];

    NewViewsRunnable(String sname, XmlDocument new_docs[]) {
      this.sname = sname;
      this.new_docs = new_docs;
    }
    public void run() {
      int index = views.indexOfTab(sname);

      XmlDocumentViews v;
      if (index == -1) {
	v = new XmlDocumentViews(sname, new_docs, text);
	views.addTab(sname, null, v);
	index = views.indexOfTab(sname);
      } else {
	v = (XmlDocumentViews)views.getComponentAt(index);
	v.setDocuments(new_docs);
      }
      v.sortViews(order);
      v.setDisplayType(display_type);
      v.setMaxSize(v.getMaxSize());
      views.setSelectedIndex(index);
      views.revalidate();
    }
  }
  public XmlViewsBase getCurrentViews() {
    Component comp = views.getSelectedComponent();
    if (comp instanceof XmlViewsBase)
      return (XmlViewsBase)comp;
    else
      return null;
  }

  public void sortViews() {
    for (int i = 0; i < views.getComponentCount(); i++) {
      Component comp = views.getComponent(i);
      if (comp instanceof XmlViewsBase) {
	XmlViewsBase v = (XmlViewsBase)comp;
	v.sortViews(order);
      }
    }
  }

  public void setDisplayType() {
    for (int i = 0; i < views.getComponentCount(); i++) {
      Component comp = views.getComponent(i);
      if (comp instanceof XmlViewsBase) {
	XmlViewsBase v = (XmlViewsBase)comp;
	v.setDisplayType(display_type);
      }
    }
  }

  public JProgressBar getProgress() { return pbar; }

  public Object getParameter(Name name) {
    String nameString = name.getNamespace();
    if (nameString == null)
      nameString = name.getLocalPart();
    else
      nameString = (nameString
		    + OutputMethodHandler.namespaceSeparator
		    + name.getLocalPart());
    return params.get(nameString);
  }

  public void setParameter(String name, Object obj) {
    params.put(name, obj);
  }

  static String[] expand(String[] args, int i) {
    Vector v = new Vector();
    try {
      FileReader fin = new FileReader(args[i].substring(1));
      BufferedReader in = new BufferedReader(fin);
      String str;

      while ((str = in.readLine()) != null) {
	if (str.length() != 0)
	  v.addElement(str);
      }
    }
    catch(Exception e) {
    }
    String[] ret = new String[args.length - 1 + v.size()];
    int j;
    for (j = 0; j < i; j++) {
      ret[j] = args[i];
    }
    for (int k = 0; k < v.size(); k++) {
      ret[j+k] = (String)v.elementAt(k);
    }
    return ret;
  }

  public static void main(String args[]) throws Exception {
      //Runtime.getRuntime().traceMethodCalls(true);
      //sun.awt.windows.WToolkit t = (sun.awt.windows.WToolkit)Toolkit.getDefaultToolkit();

    if (args.length == 0) {
      CorpusPanel p = new CorpusPanel(args.length == 0 ? "." : args[0]);
      p.addChoosableFileFilter(new ExampleFileFilter(new String[]{
	"xml"}, "XML Files"));
      p.addChoosableFileFilter(new ExampleFileFilter(new String[]{
	"xtei"}, "XML TEI Files"));
      if (p.showOpenDialog(null) == JFileChooser.CANCEL_OPTION)
	return;
      File files[] = p.getSelectedFiles();
      args = new String[files.length];
      for (int i = 0; i < files.length; i++) {
	  args[i] = files[i].getPath();
	System.out.println("Files["+i+"]="+files[i].getName());
	
      }
    }
    else {
      for (int i = 0; i < args.length; i++) {
	if (args[i].charAt(0) == '@') {
	  args = expand(args, i);
	}
      }
    }

    XmlCompare comp = new XmlCompare(args);
  }
}
