/*
 *  $Id: XmlDocumentView.java,v 1.7 2001/11/28 20:36:20 fekete Exp $
 *  Copyright (C) 1998 Jean-Daniel Fekete and Ecole des Mines de Nantes
 *
 *  This file is part of Compus.
 *  Compus is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Library General Public License as published
 *  by the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import java.awt.*;
import java.awt.image.BufferedImage;
import javax.swing.*;
import org.xml.sax.*;
import org.xml.sax.helpers.ParserFactory;
import java.util.*;
import java.io.*;

public class XmlDocumentView extends XmlViewBase {
  XmlDocument document;

  public XmlDocumentView(XmlDocument doc, ElementMap map) {
    super(map);
    document = doc;
    updateElementMap();
    text = doc.getText();
  }
  public XmlDocumentView(XmlDocument doc) {
    this(doc, null);
  }
  static public ElementMap updateElementMap(XmlDocument document, ElementMap map) {
    if (map == null)
      map = new ElementMap();
    synchronized (map) {
      for (int i = 0; i < document.elementCount(); i++) {
	XmlDocument.Element e = document.getElement(i);
	map.putNew(e.name, e.size());
	if (e.getChildCount() == 0) {
	  int offset = e.name.length() + 2; // "<"+name+">"..."</"+name+">"
	  map.updateContent(e.name,
			    document.getText().substring(e.start+offset,
							 e.end-offset-1));
	}
      }
    }
    return map;
  }
  public void setDocument(XmlDocument doc) {
    document = doc;
    elementMap = updateElementMap(document, elementMap);
    documentModified();
  }

  void updateElementMap() {
    elementMap = updateElementMap(document, elementMap);
  }

  public XmlDocument getDocument() { return document; }

  void updateSurface() {
    surface = 0;
    for (int i = 0; i < document.elementCount(); i++) {
      XmlDocument.Element e = document.getElement(i);
      Color c = elementMap.getColor(e.name);
      if (c.getAlpha() == 0)
	continue;
      surface += e.size();
    }
    //System.out.println("Surface of "+document.getName()+"="+surface);
  }

  /*
  public void swap(XmlViewBase v) {
    if (v == this) return;
    XmlDocumentView view = (XmlDocumentView)v;
    XmlDocument d = document;
    super.swap(view);
    document = view.document;
    view.document = d;
  }
  */

  protected void paintElements(Graphics g) {
    if (document.elementCount() == 0) return;
    paintNode(g, document.getElement(0), 0, getBackground());
    /*
    int end = 0;
    Vector elementStack = new Vector();
    XmlDocument.Element current_e = null;
    for (int i = 0; i < document.eventCount(); i++) {
      XmlDocument.Event ev = document.getEvent(i);
      XmlDocument.Element e = ev.getElement();
      Color c = elementMap.getColor(e.name);
      if (c.getAlpha() == 0)
	continue;
      int start = end;
      end = ev.getPosition();
      if (start != end && current_e != null) {
	addPaintRange(g, start, end, elementMap.getColor(current_e.name));
      }
      if (ev.type == XmlDocument.Event.START) {
	current_e = ev.getElement();
	elementStack.addElement(current_e);
      } else {
	elementStack.removeElementAt(elementStack.size()-1);
	if (elementStack.size() != 0)
	  current_e = (XmlDocument.Element)elementStack.lastElement();
	else
	  current_e = null;
      }
    }
    */
  }

  int paintNode(Graphics g, XmlDocument.Element node, int end, Color current) {
    Color c = elementMap.getColor(node.name);
    if (c.getAlpha() == 0) {
      c = current;
    }
    addPaintRange(g, end, node.start, current);
    end = node.start;
    for (int i = 0; i < node.getChildCount(); i++) {
      end = paintNode(g, node.getChild(i), end, c);
    }
    addPaintRange(g, end, node.end, c);
    return node.end;
  }

  void prependParents(XmlDocument.Element e, StringBuffer s) {
    if (e == null) return;
    prependParents(e.getParent(), s);
    s.append(' ');
    s.append(e.name);
  }

  public String getDocumentName() {
    return document.getName();
  }
  public String getContext(int index) {
    StringBuffer context = new StringBuffer();
    XmlDocument.Element e = document.findElementAt(index);

    context.append(getDocumentName());
    prependParents(e, context);
    return context.toString();
  }

  public static void main(String args[])
    throws Exception {
    JFrame frame = new JFrame("XmlDocumentView");
    frame.getContentPane().setLayout(new FlowLayout());
    ElementMap elementMap = null;
    Parser parser;
    InputSource source;
    FileInputStream input;
    int i;
    String defaultDriver = "com.jclark.xml.sax.Driver";
    String driver = System.getProperty("org.xml.sax.parser");
    if (driver == null)
      driver = defaultDriver;

    parser = ParserFactory.makeParser(driver);

    int maxSize = 0;
    Vector documents = new Vector();
    elementMap = new ElementMap();
    for (i = 0; i < args.length; i++) {
      XmlDocument handler = new XmlDocument(args[i]);
      XmlDocumentView view = new XmlDocumentView(handler, elementMap);
      view.setPreferredSize(new Dimension(30,500));
      frame.getContentPane().add(view);
      documents.addElement(view);
      if (handler.size() > maxSize)
	maxSize = handler.size();
    }
    for (i = 0; i < documents.size(); i++) {
      XmlDocumentView doc = (XmlDocumentView)documents.elementAt(i);
      doc.setMaxSize(maxSize);
    }
    frame.pack();
    frame.setVisible(true);
  }

  protected void collectSaveableElements(HashMap view) {
    ElementMap map = getElementMap();
    for (int i = 0; i < document.elementCount(); i++) {
	XmlDocument.Element e = document.getElement(i);
	if (e.getChildCount() == 0 &&
	    map.isVisible(e.name)) {
	  int offset = e.name.length() + 2; // "<"+name+">"..."</"+name+">"
	  addSaveableElement(view, e.name,
			     text.substring(e.start+offset,
					    e.end-offset-1));
	}
    }
  }
    

  public HashMap getView() {
    int size = getSaveableElements();
    HashMap view = new HashMap(size*2,1);

    collectSaveableElements(view);
    return view;
  }

}
