/*
 *  $Id: XmlDocumentViews.java,v 1.5 2001/11/28 20:36:20 fekete Exp $
 *  Copyright (C) 1998 Jean-Daniel Fekete and Ecole des Mines de Nantes
 *
 *  This file is part of Compus.
 *  Compus is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Library General Public License as published
 *  by the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class XmlDocumentViews extends XmlViewsBase {
  XmlDocumentView views[];

  public XmlDocumentViews(String name, XmlDocument docs[], JTextArea text) {
    super(name, text);
    views = new XmlDocumentView[docs.length];
    for (int i = 0; i < docs.length; i++) {
      initDocument(i, docs[i]);
    }
    elementMap.optimize();
  }
  public int getViewCount() { return views.length; }
  public XmlViewBase getViewAt(int index) { return views[index]; }
  protected void initDocument(int index, XmlDocument doc) {
    XmlDocumentView view = new XmlDocumentView(doc);
    // view.setBorder(BorderFactory.createEmptyBorder());
    view.setBorder(BorderFactory.createLineBorder(Color.black));
    view.setPreferredSize(pSize);
    view.addMouseListener(new MouseAdapter() {
      public void mousePressed(MouseEvent e) {
	XmlDocumentView v = (XmlDocumentView)e.getComponent();
	XmlDocument d = v.getDocument();
	if (d != text.getClientProperty(XML_DOCUMENT)) {
	  text.putClientProperty(XML_DOCUMENT, d);
	  text.setText(d.getText());
	}
	int in = v.indexOf(e.getX(), e.getY());
	System.out.println("Position: "+in+" of "+d.size());
	System.out.println("Size of text:"+text.getText().length());
	text.setCaretPosition(in);
      }
    });
    views[index] = view;
    elementMap.addElements(view.getElementMap());
    add(view, index);
  }

  public void setDocuments(XmlDocument docs[]) {
    elementMap.clear();
    if (docs.length != views.length) return; // ++ fix
    reset();
    for (int i = 0; i < docs.length; i++) {
      initDocument(i, docs[i]);
    }
    elementMap.optimize();
  }
}
