/*
 *  $Id: XmlViewsBase.java,v 1.12 2001/11/28 20:36:20 fekete Exp $
 *  Copyright (C) 1998 Jean-Daniel Fekete and Ecole des Mines de Nantes
 *
 *  This file is part of Compus.
 *  Compus is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Library General Public License as published
 *  by the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

abstract class XmlViewsBase extends JPanel 
  implements ListSelectionListener {
  //XmlXtView views[];
  protected String name;
  protected ElementMap elementMap = null;
  protected JList jlist;
  protected JTextArea text;
  public static int defaultHeight = 700;
  protected Dimension pSize = new Dimension(6, defaultHeight);
  protected XmlViewBase original_views[];
  protected int dragging_index = -1;
  protected boolean dragging_swapped;
  protected boolean selection_inverted;

  public XmlViewsBase(String name, JTextArea text) {
    super(false);
    setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
    setBorder(new RulerBorder());
    this.name = name;
    this.text = text;
    elementMap = new ElementMap();
    
    //System.out.println(elementMap);
    jlist = new JList(elementMap);
    //jlist.setPreferredSize(new Dimension(100, 800));
    jlist.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    jlist.setCellRenderer(new ElementMapRenderer(elementMap));
    jlist.setVisibleRowCount(40);
    jlist.addListSelectionListener(this);
    jlist.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
	if (e.getModifiers() == InputEvent.BUTTON3_MASK &&
	    ! dragging_swapped) {
	  jlist.requestFocus();
	  int index = jlist.locationToIndex(e.getPoint());
	  String key = (String)elementMap.getElementAt(index);
	  Color color = JColorChooser.showDialog(XmlViewsBase.this,
						 "Color Chooser",
						 elementMap.getColor(key));
	  if (color != null) {
	    elementMap.putColor(key, color);
	    XmlViewsBase.this.repaint();
	  }
	}
      }
      public void mousePressed(MouseEvent e) {
	if (e.getModifiers() == InputEvent.BUTTON3_MASK) {
	  jlist.requestFocus();
	  int index = jlist.locationToIndex(e.getPoint());
	  dragging_index = index;
	  dragging_swapped = false;
	}
      }
      public void mouseReleased(MouseEvent e) {
	if (e.getModifiers() == InputEvent.BUTTON3_MASK) {
	  dragging_index = -1;
	  dragging_swapped = false;
	}
      }
    });
    
    jlist.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseDragged(MouseEvent e) {
	if (e.getModifiers() == InputEvent.BUTTON3_MASK) {
	  int index = jlist.locationToIndex(e.getPoint());
	  if (dragging_index != index) {
	    elementMap.swapKeys(dragging_index, index);
	    dragging_index = index;
	  dragging_swapped = true;
	  }
	}
      }
    });
    JScrollPane sp =
      new JScrollPane(jlist,
		      JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
		      JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    sp.setPreferredSize(new Dimension(150, defaultHeight));
    add(sp);
  }

  public abstract int getViewCount();
  public abstract XmlViewBase getViewAt(int index);


  public void optimizeColors() {
    elementMap.optimize();
    repaint_views();
  }

  final static String XML_DOCUMENT = "XmlDocument";

  public int getMaxSize() {
    int maxSize = 0;
    for (int i = 0; i < getViewCount(); i++) {
      int size = getViewAt(i).getDocumentSize();
      if (size > maxSize)
	maxSize = size;
    }
    //System.out.println("Max size = "+maxSize);
    return maxSize;
  }
  public void setMaxSize(int max) {
    for (int i = 0; i < getViewCount(); i++) {
      getViewAt(i).setMaxSize(max);
    }
  }
  public boolean isSelectionInverted() {
    return selection_inverted;
  }
  public void setSelectionInverted(boolean b) {
    selection_inverted = b;
    valueChanged(new ListSelectionEvent(jlist,
					0, jlist.getModel().getSize()-1,
					false));
  }
  public void valueChanged(ListSelectionEvent e) {
    if (e.getValueIsAdjusting()) return;
    for (int i = e.getFirstIndex(); i <= e.getLastIndex(); i++) {
      boolean selected = jlist.isSelectedIndex(i) ?
	! selection_inverted : selection_inverted;

      if (selected) {
	Color c = getColorAt(i);
	if (c.getAlpha() != 0) {
	  //System.out.println("Erasing color #"+i);
	  c = new Color(c.getRed(), c.getGreen(), c.getBlue(), 0);
	  setColorAt(i, c);
	}
      } else {
	Color c = getColorAt(i);
	if (c.getAlpha() != 255) {
	  //System.out.println("Displaying color #"+i);
	  c = new Color(c.getRed(), c.getGreen(), c.getBlue(), 255);
	  setColorAt(i, c);
	}
      }
    }
    repaint_views();
  }

  public void repaint_views() {
    for (int i = 0; i < getViewCount(); i++)
      getViewAt(i).documentModified();
    setMaxSize(getMaxSize());
  }
  public void match(String str) {
    for (int i = 0; i < getViewCount(); i++)
      getViewAt(i).match(str);
  }

  public Color getColorAt(int index) {
    String key = (String)elementMap.getElementAt(index);
    return elementMap.getColor(key);
  }
  public void setColorAt(int index, Color color) {
    if (color != null) {
      String key = (String)elementMap.getElementAt(index);
      elementMap.putColor(key, color);
    }
  }

  public void reset() {
    for (int i = getViewCount()-1; i >= 0; i--)
      remove(i);
    original_views = null;
  }
  public void saveOriginalOrder() {
    int count = getViewCount();
    int i;
    if (original_views == null) {
      original_views = new XmlViewBase[count];
      for (i = 0; i < count; i++) {
	original_views[i] = getViewAt(i);
      }
    }
  }
  public int[] getSortOrder(Comparator comp) {
    int count = getViewCount();
    int i;
    Integer indirect[] = new Integer[count];
    for (i = 0; i < count; i++)
      indirect[i] = new Integer(i);
    saveOriginalOrder();
    if (comp != null) {
      Arrays.sort(indirect, new IndirectComparator(comp, original_views));
    }
    int ret[] = new int[count];
    for (i = 0; i < count; i++) {
      ret[i] = indirect[i].intValue();
    }
    return ret;
  }
  public int[] getSortOrder(String s) {
    return getSortOrder(getViewComp(s));
  }

  public void sortViews(int order[]) {
    saveOriginalOrder();
    int count = getViewCount();
    int i;
    for (i = count-1; i >= 0; i--) {
      remove(i);
    }
    for (i = 0; i < count; i++) {
      add(original_views[order[i]], i);
    }
    revalidate();
  }

  public void sortViews(Comparator comp) {
    //System.out.println("sortViews by "+comp);
    sortViews(getSortOrder(comp));
  }

  public void sortViews(String compName) {
    sortViews(getViewComp(compName));
  }

  public void setDisplayType(int t) {
    int count = getViewCount();
    for (int i = 0; i < count; i++) {
      getViewAt(i).setDisplayType(t);
      repaint_views();
    }
  }

  static Hashtable compTable = new Hashtable();
  static public Comparator addViewComp(String name, Comparator comp) {
    return (Comparator)compTable.put(name, comp);
  }
  static public Comparator getViewComp(String name) {
    return (Comparator)compTable.get(name);
  }
  static public String[] getViewCompNames() {
    String ret[] = new String[compTable.size()];
    int i = 0;
    for (Enumeration k = compTable.keys(); k.hasMoreElements(); ) {
      ret[i++] = (String)k.nextElement();
    }
    return ret;
  }
  static class IndirectComparator implements Comparator {
    Comparator comp;
    Object indirect_table[];

    IndirectComparator(Comparator comp, Object it[]) {
      this.comp = comp;
      indirect_table = it;
    }
    public int compare(Object o1, Object o2) {
      int i1 = ((Integer)o1).intValue();
      int i2 = ((Integer)o2).intValue();
      return comp.compare(indirect_table[i1], indirect_table[i2]);
    }
  }
  static class CompareSurface implements Comparator {
    public int compare(Object o1, Object o2) {
      XmlViewBase b1 = (XmlViewBase)o1;
      XmlViewBase b2 = (XmlViewBase)o2;
      return (int)(b1.getVisibleSurface() - b2.getVisibleSurface());
    }
  }
  static class CompareElements implements Comparator {
    public int compare(Object o1, Object o2) {
      XmlViewBase b1 = (XmlViewBase)o1;
      XmlViewBase b2 = (XmlViewBase)o2;
      return (b1.getVisibleElements() - b2.getVisibleElements());
    }
  }
  public boolean contains(int x, int y) {
      if (super.contains(x, y)) {
	  RulerBorder b = (RulerBorder)getBorder();

	  setToolTipText(b.getToolTipText(this, x, y));
	  return true;
      }
      return false;
  }

  public void saveViews(String filename) {
    try {
      FileWriter out = new FileWriter(filename);
      saveViews(out);
      out.close();
    } catch(IOException e) {
      System.out.println("IO Exception"+e);
    }
  }

  public void checkSaveableElements() {
    int elements = 0;
    ElementMap map = elementMap;
    String[] strs = map.getKeys();

    for (int i = 0; i < strs.length; i++) {
      String s = strs[i];
      ElementMap.ElementInfo info = map.getInfo(s);
      if (info.isVisible()) {
	if (info.getType() == null ||
	    info.getType().getType() == ElementType.TYPE_EMPTY) {
	  info.setVisible(false);
	}
      }
    }
  }
  
  public void saveViews(Writer out) {
    checkSaveableElements();
    String[] strs = elementMap.getKeys();
    PrintWriter pout = new PrintWriter(out);

    pout.write("# DocumentName ");
    for (int i = 0; i < strs.length; i++) {
	String s = strs[i];
	if (elementMap.isVisible(s)) {
	  pout.write(s);
	  pout.write(';');
	}
    }
    pout.write('\n');
    for (int view = 0; view < getViewCount(); view++) {
      HashMap h = getViewAt(view).getView();
      XmlViewBase v = getViewAt(view);
      // System.out.println(v.getDocumentName()+"="+h);
      pout.write(v.getDocumentName());
      pout.write(';');
      for (int i = 0; i < strs.length; i++) {
	String s = strs[i];
	if (elementMap.isVisible(s)) {
	  s =h.get(s).toString();
	  //System.out.println(s);
	  if (s != null) {
	    pout.write(s);
	  }
	  pout.write(';');
	}
      }
      pout.write('\n');
    }
  }

  static {
    addViewComp("Surface<", new CompareSurface());
    addViewComp("Surface>", new CompareSurface() {
	public int compare(Object o1, Object o2) {
	  return super.compare(o2, o1);
	}});
    addViewComp("Elements<", new CompareElements());
    addViewComp("Elements>", new CompareElements(){
	public int compare(Object o1, Object o2) {
	  return super.compare(o2, o1);
	}});
  }
}

