/*
 *  $Id: XmlXtView.java,v 1.8 2001/11/28 20:36:20 fekete Exp $
 *  Copyright (C) 1998 Jean-Daniel Fekete and Ecole des Mines de Nantes
 *
 *  This file is part of Compus.
 *  Compus is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Library General Public License as published
 *  by the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import java.awt.*;
import javax.swing.*;
import javax.swing.text.*;
import org.xml.sax.*;
import org.xml.sax.helpers.ParserFactory;
import java.util.*;
import java.io.*;
import com.jclark.xsl.sax.*;
import com.jclark.xsl.tr.*;
import com.jclark.xsl.om.*;
import java.util.Locale;
import java.io.IOException;
import java.net.URL;

public class XmlXtView extends XmlViewBase {
  Node root;
  Hashtable nodeloc;
  String name;

  static class NodeLocation {
    int start, end;

    final int size() { return end - start; }
  }


  public XmlXtView(Node root, ElementMap map) {
    super(map);
    this.root = root;
    this.name = new File(root.getURL().getFile()).getName();
    nodeloc = new Hashtable();
    StringBuffer sb = printNode(root, new StringBuffer());
    text = sb.toString();
    updateElementMap();
  }
  public XmlXtView(Node root) {
    this(root, new ElementMap());
  }
  void updateElementMap() {
    elementMap = updateElementMap(root, elementMap);
  }
  public ElementMap updateElementMap(Node node, ElementMap map) {
    if (node == null)
      return null;
    if (map == null)
      map = new ElementMap();
    switch (node.getType()) {
    case Node.ELEMENT: {
      NodeLocation loc = getNodeLoc(node);
      String name = node.getName().toString();
      map.putNew(name, loc.size());
      if (isLeaf(node)) {
	int offset = name.length() + 2; // "<"+name+">"..."</"+name+">"
	String s = text.substring(loc.start+offset,
				  loc.end-offset-1);
	//System.out.println("Creating type at "+(loc.start+offset)+
	//" and "+ (loc.end-offset-1)+
	//" contents="+s);
	map.updateContent(name, s);
      }
    }
      // Fall through
    case Node.ROOT:
      for (SafeNodeIterator iter = node.getChildren(); ; ) {
	if (updateElementMap(iter.next(), map) == null)
	  break;
      }
      break;
    }
    return map;
  }
  NodeLocation getNodeLoc(Node node) {
    return (NodeLocation)nodeloc.get(node);
  }
  public void setRootNode(Node node) {
    root = node;
    elementMap = updateElementMap(root, elementMap);
    documentModified();
    nodeloc = new Hashtable();
    StringBuffer sb = printNode(root, new StringBuffer());
    text = sb.toString();
  }

  public StringBuffer printNode(Node node, StringBuffer sb) {
    if (node == null) return null;
    else if (node.getType() == Node.TEXT) {
      //System.out.println("Text: '"+node.getData()+"'");
      sb.append(node.getData());
    }
    else if (node.getType() == Node.ELEMENT) {
      //System.out.println("Node: '"+node.getName()+"'");
      NodeLocation loc = new NodeLocation();
      loc.start = sb.length();
      sb.append("<").append(node.getName()).append(">");
      SafeNodeIterator iter = node.getChildren();
      while (printNode(iter.next(), sb) != null) ;
      sb.append("</").append(node.getName()).append(">");
      loc.end = sb.length();
      nodeloc.put(node, loc);
      return sb;
    }
    else {
      //System.out.println("Node: '"+node.getURL().toString()+"'");
      SafeNodeIterator iter = node.getChildren();
      while (printNode(iter.next(), sb) != null) ;
    }
    return sb;
  }
  public Node getRootNode() { return root; }
  protected void paintElements(Graphics g) {
    paintNode(g, root, 0, getBackground());
  }

  int paintNode(Graphics g, Node node, int end, Color current) {
    if (node.getType() == Node.ELEMENT) {
      NodeLocation loc = getNodeLoc(node);
      Color c = elementMap.getColor(node.getName().toString());
      if (c.getAlpha() == 0) {
	//System.out.println("null alpha in "+node.getName());
	c = current;
      }
      addPaintRange(g, end, loc.start, current);
      end = loc.start;
      SafeNodeIterator iter = node.getChildren();
      Node child;
      while ((child = iter.next()) != null) {
	end = paintNode(g, child, end, c);
      }
      addPaintRange(g, end, loc.end, c);
      return loc.end;
    }
    else if (node.getType() == Node.ROOT) {
      SafeNodeIterator iter = node.getChildren();
      Node child;
      while ((child = iter.next()) != null) {
	end = paintNode(g, child, end, current);
      }
    }
    return end;
  }

  /*
  public void swap(XmlViewBase v) {
    if (v == this) return;
    XmlXtView view = (XmlXtView)v;
    Node r = root;
    Hashtable nl = nodeloc;
    super.swap(view);
    root = view.root;
    nodeloc = view.nodeloc;
    view.root = r;
    view.nodeloc = nl;
  }

  void updateSurface() {
    surface = surfaceNode(root);
    System.out.println("Surface of "+root.getURL()+"="+surface);
  }
  long surfaceNode(Node node) {
    long size = 0;
    switch(node.getType()) {
    case Node.ELEMENT: {
      NodeLocation loc = getNodeLoc(node);
      Color c = elementMap.getColor(node.getName().toString());
      
      if (c.getAlpha() != 0) {
	//System.out.println("null alpha in "+node.getName());
	size = loc.end - loc.start + 1;
      }
    }
    // fall through
    case Node.ROOT: {
      SafeNodeIterator iter = node.getChildren();
      Node child;
      while ((child = iter.next()) != null) {
	size += surfaceNode(child);
      }
    }
    }
    return size;
  }
  */

  void prependParents(Node e, StringBuffer s) {
    if (e == null) return;
    prependParents(e.getParent(), s);
    s.append(' ');
    s.append(e.getName());
  }

  public String getDocumentName() {
    return name;
  }

  public String getContext(int index) {
    StringBuffer context = new StringBuffer();
    Node e = findElementAt(index);

    context.append(getDocumentName());
    prependParents(e, context);
    return context.toString();
  }

  Node findElementAt(int index) {
    return moreSpecific(root, index);
  }

  boolean contains(Node e, int index) {
    switch(e.getType()) {
    case Node.ROOT:
      return true;
    case  Node.ELEMENT: {
      NodeLocation l = (NodeLocation)nodeloc.get(e);
      return (l.start <= index) && (index <= l.end);
    }
    }
    return false;
  }
  Node moreSpecific(Node e, int index) {
    if (! contains(e, index)) return null;
    SafeNodeIterator iter = e.getChildren();
    Node child;
    while ((child = iter.next()) != null) {
      if (child.getType() != Node.ELEMENT)
	continue;
      Node ce = moreSpecific(child, index);
      if (ce != null)
	return ce;
    }
    return e;
  }
  
  public String toString() {
    return "XmlXtView of "+root.getURL();
  }

  static final boolean isLeaf(Node node) {
    SafeNodeIterator iter = node.getChildren();
    Node child;
    while ((child = iter.next()) != null) {
      if (child.getType() == Node.ELEMENT)
	return false;
    }
    return true;
  }

  protected void collectSaveableElements(HashMap view, Node node) {
    //System.out.println("Node: "+node);
    ElementMap map = getElementMap();
    if (node == null)
      return;
    switch(node.getType()) {
    case Node.ELEMENT:
      NodeLocation loc = getNodeLoc(node);
      String name = node.getName().toString();
      ElementMap.ElementInfo info = map.getInfo(name);
      if (info != null && info.isVisible()) {
	int offset = name.length() + 2; // "<"+name+">"..."</"+name+">"
	String s = text.substring(loc.start+offset,
				  loc.end-offset-1);
	addSaveableElement(view, name, s);
      }
      // Fall through
    case Node.ROOT:
      for (SafeNodeIterator iter = node.getChildren(); ; ) {
	Node n = iter.next();
	if (n == null)
	    break;
	collectSaveableElements(view, n);
      }
      break;
    }
  }

  public HashMap getView() {
    int size = getSaveableElements();
    HashMap view = new HashMap(size*2,1);

    collectSaveableElements(view, root);
    return view;
  }
}
  
